/* Copyright 2012 Tobias Marschall
 *
 * This file is part of CLEVER.
 *
 * CLEVER is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * CLEVER is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with CLEVER.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ASTAGCALCULATOR_H_
#define ASTAGCALCULATOR_H_

#include <iostream>
#include <boost/unordered_map.hpp>
#include <bamtools/api/BamReader.h>

#include "NamedDnaSequence.h"

class ASTagCalculator {
private:
	boost::unordered_map<std::string,NamedDnaSequence*>* reference_sequences;

	long long mapped_reads;
	long long reference_missing;
	long long bad_alignments;
	int bad_score_threshold;
	int phred_offset;
	long long overhanging_alignments;
public:
	ASTagCalculator(const std::string& reference_filename, int phred_offset, int bad_score_threshold);
	virtual ~ASTagCalculator();

	void computeTag(BamTools::BamAlignment& read_aln, const BamTools::RefVector& bam_ref_data);
	void printWarnings(std::ostream& os);
};

#endif /* ASTAGCALCULATOR_H_ */
